/* can_structures.h
 *
 * Command structure definitions
 *
 * This file can be used by Lua FFI.
 *
 */

#pragma pack(push)
#pragma pack(1)

typedef enum { CAN_TS_OFFSET = 1, CAN_GET_SYNC_TS, CAN_GET_TS,
               CAN_SETUP_SYNC_TS, CAN_PASSTHRU_POWER_ON, CAN_PASSTHRU_POWER_OFF,
               CAN_ENABLE_TERMINATOR, CAN_DISABLE_TERMINATOR,
               CAN_SETUP, CAN_RX_START, CAN_RX_STOP,
               CAN_TX_START, CAN_TX_MSG, CAN_TX_STOP,
               CAN_EVENT_SETUP, CAN_EVENT_START, CAN_EVENT_STOP,
               CAN_LAST_COMMAND = 0x7FFFFFFF    /* Insures 32-bit enum */
} can_cmd_t;

typedef struct {
    uint8_t                     sjw;                /* 1 to 4 */
    uint8_t                     brp;                /* 1 to 64 */
    uint8_t                     tseg1;              /* 1 to 16 */
    uint8_t                     tseg2;              /* 1 to 8 */
    uint8_t                     samples_per_bit;    /* 1 or 3 */
    uint8_t                     listen_only;        /* 0 or 1 */
    uint8_t                     reserved[2];
} can_cmd_setup_s;

typedef struct {
    int64_t                     start_time;
    cmd_rate_parameters_s       heartbeat_interval;
    uint32_t                    phys_address;
} can_cmd_rx_start_s;

typedef struct {
    uint32_t                    phys_address;
    uint32_t                    channel_id;
} can_cmd_tx_start_stop_s;

typedef struct {
    uint8_t         bus_off_deassert;
    uint8_t         bus_off;
    uint8_t         reserved[2];
} can_cmd_event_vector_s;

typedef struct {
    int64_t     ts;
    uint32_t    can_id;
    uint8_t     can_message[8];
    uint8_t     frame_format;
    uint8_t     tx_error_count;
    uint8_t     rx_error_count;
    uint8_t     error_capture_code;
} can_rxtx_buffer_t;

typedef struct {
    can_cmd_t       cmd;
    uint32_t        cmd_id;
    union {
        cmd_ts_offset_s         ts_offset;
        can_cmd_setup_s         can_setup;
        can_cmd_rx_start_s      rx_start;
        can_cmd_tx_start_stop_s tx_start_stop;
        can_rxtx_buffer_t       tx_msg;
        cmd_event_start_s       ev_start;
        can_cmd_event_vector_s  events;
    } prm;
} can_cmd_s;

typedef struct {
    int64_t                     ts;
    can_cmd_event_vector_s      events;
} can_ev_buffer_t;

enum { MAX_CAN_TX_STARTS = 16 };

#pragma pack(pop)
