local ffi = require 'ffi'
local lj_include = require 'lj_include'

lj_include('signal')
lj_include('bits_sigcontext')

ffi.cdef[[
typedef unsigned long gregset_t[48];
typedef struct _libc_fpstate {
    double fpregs[32];
    double fpscr;
    unsigned int _pad[2];
} fpregset_t;
typedef struct _libc_vrstate {
    unsigned int vrregs[32][4];
    unsigned int vrsave;
    unsigned int _pad[2];
    unsigned int vscr;
} vrregset_t;
typedef struct {
    gregset_t gregs;
    fpregset_t fpregs;
    vrregset_t vrregs;
} mcontext_t;
typedef struct ucontext {
    unsigned long int uc_flags;
    struct ucontext *uc_link;
    stack_t uc_stack;
    int uc_pad[7];
    union uc_regs_ptr {
	struct pt_regs *regs;
	mcontext_t *uc_regs;
    } uc_mcontext;
    sigset_t uc_sigmask;
    char uc_reg_space[sizeof(mcontext_t) + 12];
}
ucontext_t;
]]

local defs = {
    NGREG = '48',
    NFPREG = '33',
    NVRREG = '34',
}
local format, cdef, C = string.format, ffi.cdef, ffi.C
for sym, val in pairs(defs) do
    if not pcall(function() return C[sym] end) then
        cdef(format("enum {%s = %s};", sym, val))
    end
end

