require 'strict'

local ffi = require 'ffi'
local libpcre = ffi.load('libpcre.so.0')

ffi.cdef[[

enum {
    PCRE_MAJOR = 8,
    PCRE_MINOR = 10,
    PCRE_CASELESS = 0x00000001,
    PCRE_MULTILINE = 0x00000002,
    PCRE_DOTALL = 0x00000004,
    PCRE_EXTENDED = 0x00000008,
    PCRE_ANCHORED = 0x00000010,
    PCRE_DOLLAR_ENDONLY = 0x00000020,
    PCRE_EXTRA = 0x00000040,
    PCRE_NOTBOL = 0x00000080,
    PCRE_NOTEOL = 0x00000100,
    PCRE_UNGREEDY = 0x00000200,
    PCRE_NOTEMPTY = 0x00000400,
    PCRE_UTF8 = 0x00000800,
    PCRE_NO_AUTO_CAPTURE = 0x00001000,
    PCRE_NO_UTF8_CHECK = 0x00002000,
    PCRE_AUTO_CALLOUT = 0x00004000,
    PCRE_PARTIAL_SOFT = 0x00008000,
    PCRE_DFA_SHORTEST = 0x00010000,
    PCRE_DFA_RESTART = 0x00020000,
    PCRE_FIRSTLINE = 0x00040000,
    PCRE_DUPNAMES = 0x00080000,
    PCRE_NEWLINE_CR = 0x00100000,
    PCRE_NEWLINE_LF = 0x00200000,
    PCRE_NEWLINE_CRLF = 0x00300000,
    PCRE_NEWLINE_ANY = 0x00400000,
    PCRE_NEWLINE_ANYCRLF = 0x00500000,
    PCRE_BSR_ANYCRLF = 0x00800000,
    PCRE_BSR_UNICODE = 0x01000000,
    PCRE_JAVASCRIPT_COMPAT = 0x02000000,
    PCRE_NO_START_OPTIMIZE = 0x04000000,
    PCRE_NO_START_OPTIMISE = 0x04000000,
    PCRE_PARTIAL_HARD = 0x08000000,
    PCRE_NOTEMPTY_ATSTART = 0x10000000,
    PCRE_UCP = 0x20000000,
    PCRE_INFO_OPTIONS = 0,
    PCRE_INFO_SIZE = 1,
    PCRE_INFO_CAPTURECOUNT = 2,
    PCRE_INFO_BACKREFMAX = 3,
    PCRE_INFO_FIRSTBYTE = 4,
    PCRE_INFO_FIRSTTABLE = 5,
    PCRE_INFO_LASTLITERAL = 6,
    PCRE_INFO_NAMEENTRYSIZE = 7,
    PCRE_INFO_NAMECOUNT = 8,
    PCRE_INFO_NAMETABLE = 9,
    PCRE_INFO_STUDYSIZE = 10,
    PCRE_INFO_DEFAULT_TABLES = 11,
    PCRE_INFO_OKPARTIAL = 12,
    PCRE_INFO_JCHANGED = 13,
    PCRE_INFO_HASCRORLF = 14,
    PCRE_INFO_MINLENGTH = 15,
    PCRE_CONFIG_UTF8 = 0,
    PCRE_CONFIG_NEWLINE = 1,
    PCRE_CONFIG_LINK_SIZE = 2,
    PCRE_CONFIG_POSIX_MALLOC_THRESHOLD = 3,
    PCRE_CONFIG_MATCH_LIMIT = 4,
    PCRE_CONFIG_STACKRECURSE = 5,
    PCRE_CONFIG_UNICODE_PROPERTIES = 6,
    PCRE_CONFIG_MATCH_LIMIT_RECURSION = 7,
    PCRE_CONFIG_BSR = 8,
    PCRE_EXTRA_STUDY_DATA = 0x0001,
    PCRE_EXTRA_MATCH_LIMIT = 0x0002,
    PCRE_EXTRA_CALLOUT_DATA = 0x0004,
    PCRE_EXTRA_TABLES = 0x0008,
    PCRE_EXTRA_MATCH_LIMIT_RECURSION = 0x0010,
    PCRE_EXTRA_MARK = 0x0020
};
struct real_pcre;
typedef struct real_pcre pcre;
typedef struct pcre_extra {
    unsigned long int flags;
    void *study_data;
    unsigned long int match_limit;
    void *callout_data;
    const unsigned char *tables;
    unsigned long int match_limit_recursion;
    unsigned char **mark;
} pcre_extra;
typedef struct pcre_callout_block {
    int version;
    int callout_number;
    int *offset_vector;
    const char *subject;
    int subject_length;
    int start_match;
    int current_position;
    int capture_top;
    int capture_last;
    void *callout_data;
    int pattern_position;
    int next_item_length;
} pcre_callout_block;
extern void *(*pcre_malloc) (size_t);
extern void (*pcre_free) (void *);
extern void *(*pcre_stack_malloc) (size_t);
extern void (*pcre_stack_free) (void *);
extern int (*pcre_callout) (pcre_callout_block *);
extern pcre *pcre_compile(const char *, int, const char **, int *,
    const unsigned char *);
extern pcre *pcre_compile2(const char *, int, int *, const char **,
    int *, const unsigned char *);
extern int pcre_config(int, void *);
extern int pcre_copy_named_substring(const pcre *, const char *,
    int *, int, const char *, char *, int);
extern int pcre_copy_substring(const char *, int *, int, int, char *, int);
extern int pcre_dfa_exec(const pcre *, const pcre_extra *,
    const char *, int, int, int, int *, int, int *, int);
extern int pcre_exec(const pcre *, const pcre_extra *, const char *,
    int, int, int, int *, int);
extern void pcre_free_substring(const char *);
extern void pcre_free_substring_list(const char **);
extern int pcre_fullinfo(const pcre *, const pcre_extra *, int, void *);
extern int pcre_get_named_substring(const pcre *, const char *,
    int *, int, const char *, const char **);
extern int pcre_get_stringnumber(const pcre *, const char *);
extern int pcre_get_stringtable_entries(const pcre *, const char *,
    char **, char **);
extern int pcre_get_substring(const char *, int *, int, int,
    const char **);
extern int pcre_get_substring_list(const char *, int *, int,
    const char ***);
extern int pcre_info(const pcre *, int *, int *);
extern const unsigned char *pcre_maketables(void);
extern int pcre_refcount(pcre *, int);
extern pcre_extra *pcre_study(const pcre *, int, const char **);
extern const char *pcre_version(void);

]]

-- Return a table with both the ffi instance and the library reference
return {ffi, libpcre}
