local ffi = require'ptpv2-ffi/cdata'
local C = ffi.C

InternalTime = {}

function InternalTime:new(s, ns)
    local Internal_time = {
        time = ffi.new('internal_time_t'),
    }
    Internal_time.time.seconds = s
    Internal_time.time.nanoseconds = ns
    setmetatable(Internal_time, self)
    self.__index = self
    return Internal_time
end

function InternalTime:set_time(s, ns)
    self.time.seconds = s
    self.time.nanoseconds = ns
end

function InternalTime:get_time()
    return self.time.seconds, self.time.nanoseconds
end

local function normalize_t(s, ns)
    while ns <= -1000000000 do
        ns = ns + 1000000000
        s = s - 1
    end
    while ns >= 1000000000 do
        ns = ns - 1000000000
        s = s + 1
    end
    if s < 0 and ns > 0 then
        ns = ns - 1000000000
        s = s + 1
    elseif s > 0 and ns < 0 then
        ns = ns + 1000000000
        s = s - 1
    end
    return s, ns
end

function InternalTime:__add(a)
    return InternalTime:new(normalize_t(
        self.time.seconds + a.time.seconds,
        self.time.nanoseconds + a.time.nanoseconds))
end

function InternalTime:__sub(a)
    return InternalTime:new(normalize_t(
        self.time.seconds - a.time.seconds,
        self.time.nanoseconds - a.time.nanoseconds))
end

function InternalTime:__div(a)
    local t = (self.time.seconds * 1000000000) + self.time.nanoseconds
    t = t / a
    return InternalTime:new(t / 1000000000, t % 1000000000)
end

function InternalTime:__tostring()
--[[
    return string.format("%d",
        (self.time.seconds * 1000000000) + self.time.nanoseconds)
]]
    local s, ns = normalize_t(self.time.seconds, self.time.nanoseconds)
    return string.format("%d.%09d", s, ns)
end

function InternalTime:to_timestamp()
--    local timestamp = ffi.new('timestamp_t')
    local timestamp = ffi.cast('timestamp_t *', ffi.new('char[128]'))

    if bit.band(self.time.seconds, 0x80000000) == 0x80000000 or
        bit.band(self.time.nanoseconds, 0x80000000) == 0x80000000 then
        return
    end
    timestamp.seconds.msb = 0
    timestamp.seconds.lsb = self.time.seconds
    timestamp.nanoseconds = self.time.nanoseconds
    return timestamp
end
