local ffi = require'ptpv2-ffi/cdata'
local C = ffi.C

local m = {}

m.from_state_func = {}

function m:from_state_master()
    self.sync_interval_timer:stop(self.loop)
    self.announce_interval_timer:stop(self.loop)
    if self.pdelay_request_interval_timer then
        self.pdelay_request_interval_timer:stop(self.loop)
    end
end

function m:from_state_slave()
    self.announce_receipt_timer:stop(self.loop)
    if self.delay_request_interval_timer then
        self.delay_request_interval_timer:stop(self.loop)
    end
    if self.pdelay_request_interval_timer then
        self.pdelay_request_interval_timer:stop(self.loop)
    end
    self:init_clock()
end

function m:from_state_passive()
    self.pdelay_request_interval_timer:stop(self.loop)
    self.announce_receipt_timer:stop(self.loop)
end

function m:from_state_listening()
    self.announce_receipt_timer:stop(self.loop)
end

m.from_state_func[C.PTP_STATE_MASTER] = m.from_state_master
m.from_state_func[C.PTP_STATE_SLAVE] = m.from_state_slave
m.from_state_func[C.PTP_STATE_PASSIVE] = m.from_state_passive
m.from_state_func[C.PTP_STATE_LISTENING] = m.from_state_listening

m.to_state_func = {}

function m:to_state_initializing(ci)
    ci.state = C.PTP_STATE_INITIALIZING
end

function m:to_state_faulty(ci)
    ci.state = C.PTP_STATE_FAULTY
os.exit(1)
end

function m:to_state_disabled(ci)
    ci.state = C.PTP_STATE_DISABLED
end

function m:to_state_listening(ci)
    if not self.announce_receipt_timer:is_active() then
        self.announce_receipt_timer:start(self.loop)
    end
    local t = math.pow(2, ci.log_announce_interval) *
        ci.announce_receipt_timeout
    self.announce_receipt_timer:again(self.loop, t)
    ci.state = C.PTP_STATE_LISTENING
end

function m:to_state_master(ci)
    if not self.sync_interval_timer:is_active() then
        self.sync_interval_timer:start(self.loop)
    end
    local t = math.pow(2, ci.log_sync_interval)
    self.sync_interval_timer:again(self.loop, t)

    if not self.announce_interval_timer:is_active() then
        self.announce_interval_timer:start(self.loop)
    end
    local t = math.pow(2, ci.log_announce_interval)
    self.announce_interval_timer:again(self.loop, t)

    if self.pdelay_req_interval_timer then
        if not self.pdelay_req_interval_timer:is_active() then
            self.pdelay_req_interval_timer:start(self.loop)
        end
        local t = math.pow(2, ci.log_announce_interval)
        self.pdelay_req_interval_timer:again(self.loop, t)
    end
    ci.state = C.PTP_STATE_MASTER
end

function m:to_state_passive(ci)
    ci.state = C.PTP_STATE_PASSIVE

    if not self.pdelay_req_interval_timer:is_active() then
        self.pdelay_req_interval_timer:start(self.loop)
    end
    local t = math.pow(2, ci.log_announce_interval)
    self.pdelay_req_interval_timer:again(self.loop, t)

    if not self.announce_receipt_timer:is_active() then
        self.announce_receipt_timer:start(self.loop)
    end
    local t = math.pow(2, ci.log_announce_interval) *
        ci.announce_receipt_timeout
    self.announce_receipt_timer:again(self.loop, t)
    ci.state = C.PTP_STATE_PASSIVE
end

function m:to_state_uncalibrated(ci)
    ci.state = C.PTP_STATE_UNCALIBRATED
end

function m:to_state_slave(ci)
    self:init_clock()

    ci.waiting_for_follow = 0
    self.pdelay_req_send_time.seconds = 0
    self.pdelay_req_send_time.nanoseconds = 0

    self.pdelay_req_receive_time.seconds = 0
    self.pdelay_req_receive_time.nanoseconds = 0

    self.pdelay_resp_send_time.seconds = 0
    self.pdelay_resp_send_time.nanoseconds = 0

    self.pdelay_resp_receive_time.seconds = 0
    self.pdelay_resp_receive_time.nanoseconds = 0

    if not self.announce_receipt_timer:is_active() then
        self.announce_receipt_timer:start(self.loop)
    end
    local t = math.pow(2, ci.log_announce_interval) *
        ci.announce_receipt_timeout
    self.announce_receipt_timer:again(self.loop, t)

    if self.pdelay_request_interval_timer then
        if not self.pdelay_request_interval_timer:is_active() then
            self.pdelay_request_interval_timer:start(self.loop)
        end
        local t = math.pow(2, ci.log_min_delay_req_interval)
        self.pdelay_request_interval_timer:again(self.loop, t)
    else
        if not self.delay_request_interval_timer:is_active() then
            self.delay_request_interval_timer:start(self.loop)
        end
        local t = math.pow(2, ci.log_min_delay_req_interval)
        self.delay_request_interval_timer:again(self.loop, t)
    end
    ci.state = C.PTP_STATE_SLAVE
end

m.to_state_func[C.PTP_STATE_INITIALIZING] = m.to_state_initializing
m.to_state_func[C.PTP_STATE_FAULTY] = m.to_state_faulty
m.to_state_func[C.PTP_STATE_DISABLED] = m.to_state_disabled
m.to_state_func[C.PTP_STATE_LISTENING] = m.to_state_listening
m.to_state_func[C.PTP_STATE_MASTER] = m.to_state_master
m.to_state_func[C.PTP_STATE_PASSIVE] = m.to_state_passive
m.to_state_func[C.PTP_STATE_UNCALIBRATED] = m.to_state_uncalibrated
m.to_state_func[C.PTP_STATE_SLAVE] = m.to_state_slave

function m:to_state(state)
    local clock_info = self.clock_info

    clock_info.message_activity = 1
    local from_state_func = m.from_state_func[clock_info.state]
    if from_state_func then from_state_func(self) end
    local to_state_func = m.to_state_func[state]
    if to_state_func then
        to_state_func(self, clock_info)
        if self.stat_cb then
            self.stat_cb(self:get_stats())
        end
    else
        self:log("error", string.format("Unknown state: %d", state))
    end
end

return m
