require 'strict'

local ffi = require 'ffi'

dbg = function() end

local function max(a, b) return a > b and a or b end
local function min(a, b) return a < b and a or b end

local mt = {}

mt.min_stamp = bit.lshift(1LL, 63)
mt.max_stamp = bit.bnot(mt.min_stamp)

function mt:initialize(start_stamp, stop_stamp)
    self.start_stamp = start_stamp and start_stamp or self.min_stamp
    self.stop_stamp = stop_stamp and stop_stamp or self.max_stamp
end

function mt:is_in_bracket(stamp)
    self.in_count = self.in_count + 1
    if stamp >= self.start_stamp and stamp <= self.stop_stamp then
        self.out_count = self.out_count + 1
        return true
    elseif stamp < self.start_stamp then
        return false
    else
        if self.stopped == 0 then
            self.stopped = 1
        end
        return nil
    end
end

function mt:bracket_sequential(stamp, offset, count)
    self.in_count = self.in_count + count
    if stamp + offset >= self.start_stamp
            and stamp + count - 1 <= self.stop_stamp then
        self.out_count = self.out_count + count
        return offset, count
    end
    if stamp + offset > self.stop_stamp then
        if self.stopped == 0 then
            self.stopped = 1
        end
        return 1, 0
    end
    if stamp + count - 1 < self.start_stamp then
        return 1, 0
    end

    local first_stamp = max(self.start_stamp, stamp + offset)
    local last_stamp = min(self.stop_stamp, stamp + count - 1)
    local ok_count = last_stamp - first_stamp + 1

    if ok_count > 0 and ok_count <= count - offset then
        if self.stopped == 0 and last_stamp <= stamp + count - 1 then
            self.stopped = 1
        end
        self.out_count = self.out_count + ok_count
        return tonumber(first_stamp - stamp), tonumber(ok_count)
    else
        return 1, 0
    end
end

ffi.cdef[[
    struct time_bracket {
        int64_t start_stamp;    // Count
        int64_t stop_stamp;
        int64_t current_stamp;
        int64_t in_count;
        int64_t out_count;
        int started;
        int stopped;
        double sample_rate; // KLUDGE s.b. rational!!!
    };
]]

local ctor = ffi.metatype('struct time_bracket', {__index = mt})

local m = {}

function m:new(start_stamp, stop_stamp, sample_rate)
    local obj = ctor()
    obj:initialize(start_stamp, stop_stamp)
    obj.sample_rate = sample_rate or 0
    return obj
end

return m
