/*jshint white:false */
(function( $, undefined ) {

$.widget( 'ui.somat_dial',
{
    /**
     * @description This attribute defines the natural pixel width of the
     *              gauge. It is used for scale math.
     */
    _normalW     :  100,
    /**
     * @description This attribute defines the natural pixel height of the
     *              gauge. It is used for scale math.
     */
    _normalH     :  110,
    /**
     * @description This attribute defines the degree rotation of the
     *              gauge needle. It is used for percentage math.
     */
    _degrees     :  225,

    _baseURL     :  $('script:last').prop('src').replace(/[^\/]*$/, ''),

    // private
    'ready'       :      false,
    'scale_x'     :        1.0,
    'scale_y'     :        1.0,
    'range'       :        100,

    // layout
    'lowerX'      :         28,
    'lowerY'      :         74,
    'upperX'      :         75,
    'upperY'      :         74,
    'unitX'       :         50,
    'unitY'       :         83,
    'legendX'     :         50,
    'legendY'     :        105,
    'rangeFont'   :         40,
    'unitFont'    :         60,
    'legendFont'  :         40,

    // dom
    'svg'         :       null,
    'needle'      :       null,

    /**
     * @description This object stores the instance data of this widget.
     */
    options: {
        'skin'        :       'white',
        'cull'        :          true,
        'top'         :             0,
        'left'        :             0,
        'height'      :           220,
        'width'       :           200,
        'legend'      :    'abcd1234',
        'unit'        :         'psi',
        'range_min'   :             0,
        'range_max'   :           100,
        'reading'     :             0,
        'ticks_major' :          null,
        'ticks_minor' :          null,
        'readout_range_min' :    null,
        'readout_range_max' :    null
    },

    /**
     * @description This operation primes the options object for use.
     */
    _prime: function() {
        var options = this.options;
        // update the readouts and range.
        if (options.readout_range_min === null)
            options.readout_range_min = options.range_min;
        if (options.readout_range_max === null)
            options.readout_range_max = options.range_max;

        this.range = options.readout_range_max - options.readout_range_min;

        // normalize the scale of the gauge.
        this.scale_x = options.width / this._normalW;
        this.scale_y = options.height / this._normalH;

        // configure the ticks of the gauge.
        if (options.ticks_major === null)
            options.ticks_major = this.range / 10.0;
        if (options.ticks_minor === null)
            options.ticks_minor = options.ticks_major / 5.0;
    },

    /**
     * @description This operation initializes the runtime data and
     *              initiates the asyncronous loading of the gauge GUI..
     */
    _create: function() {
        var options = this.options;

        // prime the options passed in.
        this._prime();

        // update the DOM element for this gauge.
        var self = $(this.element[0]);
        self.addClass('dial_gauge').css({
            // top: options.top + 'px',
            // left: options.left + 'px',
            width: options.width + 'px',
            height: options.height + 'px'
        });

        // load in the gauge graphic.
        var handle = this;
        self.svg({
            loadURL: this._baseURL + 'svg/simple_dial_gauge.svg',
            onLoad: function(svg, error) {
                handle._ready(svg, error);
            }
        });
    },

    /**
     * @private
     * @description This operation applies any final operations to the DOM
     *              in preparation for runtime use. It expects the return
     *              signature produced by the SVG plug-in callback.
     * @param svg This argument provides access to the svg object.
     * @param error This argument, when defined, indicates an error.
     */
    _ready: function(svg, error) {
        var options = this.options;
        if (typeof(error) !== 'undefined') {
            // if (typeof(console.log) !== 'undefined')
            //    console.log(error);
            return;
        }

        var root = svg.root();
        this.svg = svg;
        this.needle = $('.gaugeNeedle', root)[0];

        if (options.skin === 'white') {
            $('circle.gaugeMask', root).attr('style',
                'fill:url(#radial_light_mat)');
            $('rect.gaugeMask', root).attr('style',
                'fill:url(#light_mat)');
            $('.gaugeFont', root).attr('fill', '#0c0c0d');
        }

        this._drawTicks(); // render the guage ticks
        this.ready = true;
        this._refresh();
    },

    _drawTicks: function() {
        var options = this.options;
        var dom_svg = this.svg;
        var ticks = $('.gaugeTicks' , dom_svg.root());
        
        // remove previous goo.
        ticks.empty();

        var x, g;
        var tick_opts = {'class': 'ticks_' + options.skin};
        if (options.ticks_minor) {
            for (x = options.readout_range_min;
                 x <= options.readout_range_max;
                 x += options.ticks_minor)
             {
                // provide a modulo with a resolution of 0.000
                var modulo = Math.round((x - options.readout_range_min) * 1000.0) %
                             (options.ticks_major * 1000.0);
                if (modulo)
                {
                    g = dom_svg.group(ticks, {
                        transform: 'rotate(' + (this._rotation(x)) + ')'
                    });
                    dom_svg.rect(g, -38, -0.5, 5, 1, tick_opts);
                }
            }
        }
        if (options.ticks_major) {
            for (x = options.readout_range_min; x <= (options.readout_range_max + options.ticks_minor); x += options.ticks_major) {
                g = dom_svg.group(ticks, {
                    'transform' : 'rotate(' + (this._rotation(x)) + ')'});
                dom_svg.rect(g, -38, -0.5, 10, 1, tick_opts);
            }
        }
    },

    /**
     * @private
     * @description This operation takes no arguments. It will update all
     *              GUI elements of the SVG Gauge. It will have no effect
     *              if '_ready' is FALSE.
     */
    _refresh: function() {
        if (!this.ready)
            return;
        var options = this.options;
        var svg_root = this.svg.root();
        var scale_x = this.scale_x;
        var scale_y = this.scale_y;
        var lowerScale = Math.min(scale_x, scale_y);

        // update the rendering of the gauge.
        $('.gauge', svg_root).attr(
            'transform', 'scale(' + scale_x + ',' + scale_y + ')'
        );
        $('.gaugeLower', svg_root).text(options.readout_range_min).attr({
            'x'         : this.lowerX * scale_x,
            'y'         : this.lowerY * scale_y,
            'font-size' : (this.rangeFont * lowerScale) + '%'
        });
        $('.gaugeUpper', svg_root).text(options.readout_range_max).attr({
            'x'         : this.upperX * scale_x,
            'y'         : this.upperY * scale_y,
            'font-size' : (this.rangeFont * lowerScale) + '%'
        });
        $('.gaugeUnit', svg_root).text(options.unit).attr({
            'x'         : this.unitX * scale_x,
            'y'         : this.unitY * scale_y,
            'font-size' : (this.unitFont * lowerScale) + '%'
        });
        $('.gaugeLegend', svg_root).text(options.legend).attr({
            'x'         : this.legendX * scale_x,
            'y'         : this.legendY * scale_y,
            'font-size' : (this.legendFont * lowerScale) + '%'
        });
        this.set_reading(options.reading);
    },

    /**
     * @private
     * @description This operation calculates the rotation of the needle.
     * @return This operation returns a value in degrees where 'min' = '0'
     *         degrees and 'max' = 'degrees' degrees.
     */
    _rotation: function(reading) {
        var options = this.options;
        return ((reading - options.readout_range_min) / this.range) * this._degrees;
    },

    /**
     * @description This operation provides the default size of this gauge.
     */
    get_default_size: function() {
        return {height: this._normalH, width: this._normalW};
    },

    /**
     * @description This operation provides access to this widgets options
     *              to the outside world.
     */
    get_configuration: function() {
        return this.options;
    },

    /**
     * @description This operation updates the current configuration of
     *              the gauge and refreshes the display.
     * @param new_config This argument expects and object containing key
     *                   value pairs to be updated. Unspecified
     *                   configuration items will not be changed.
     */
    set_configuration: function( new_config ) {
        if (new_config) {
            var options = this.options;
            // Merge in the new configuration items.
            $.extend(this.options, new_config);
            // prime the options passed in.
            this._prime();

            // skip past render updates if not ready.
            if (this.ready)
            {
                // update the root to the new size.
                $('.gauge', this.svg.root()).parent().attr({
                    height: options.height,
                    width: options.width
                });
                this._drawTicks();
            }

            // redraw the gauge.
            this._refresh();
        }
    },

    /**
     * @description This operation updates the current reading of the
     *              gauge and updates the GUI when '_ready' is TRUE.
     * @param updated_reading This argument defines the new FLOAT value
     *                        that this gauge should reflect.
     */
    set_reading: function(updated_reading) {
        var options = this.options;
        var updated = parseFloat(updated_reading);
        if (options.cull) {
            // peg the needle.
            if (updated < options.readout_range_min)
                updated = options.readout_range_min;
            else if (updated > options.readout_range_max)
                updated = options.readout_range_max;
        }
        // apply the value.
        options.reading = updated;
        if (this.ready) {
            this.needle.setAttribute(
                'transform', 'rotate(' + this._rotation(updated) + ')');
        }
    },

    destroy: function() {
        var self = $(this.element[0]);
        self.svg('destroy');
        self.removeClass('dial_gauge');

        $.Widget.prototype.destroy.apply(this, arguments);
    }
});

}( jQuery ));
