(function($) {
    "use strict";
    /*globals Slick */

    var backchannel_fields = {
        output_state: true,
        unit_identify: true,
        update_rate: true,
        power: true, // KLUDGE NAME
        channel_mode: true // KLUDGE NAME
    };

    var error_priorities = {
        error: 0,
        warning: 1,
        notice: 2 // advisory???
    };
    /*
    var priority_errors = {
        0: "error",
        1: "warning",
        2: "notice"
    };
    */
    var error_classes = {
        0: 'armada-error-cell',
        1: 'armada-error-row',
        10: 'armada-warning-cell',
        11: 'armada-warning-row',
        20: 'armada-notice-cell',
        21: 'armada-notice-row'
    };

    // Error scopes: cell, row (item), document (inter-row)
    var cell_checks = {
        collect: [{
            level: 'warning',
            message_tr: 'This channel is not used or stored',
            regexp: /^true$/
        }],
        name: [{
            level: 'error',
            message_tr: 'Name must exist',
            regexp: /^\s*\S+\s*$/
        }],
        sample_rate: [{
            level: 'error',
            message_tr: 'Sample rate must be numeric',
            regexp: /^(\s*(\d+|\d+\s*\/\s*\d+|\d+\.\d+)\s*$|$)/
        }],
        filter_freq: [{
            level: 'error',
            message_tr: 'Filter frequency must be numeric',
            regexp: /^(\s*(\d+|\d+\s*\/\s*\d+|\d+\.\d+)\s*$|$)/
        }],
        filter_charac: [{
            level: 'error',
            message_tr: 'Filter characteristics must be Bessel or Butterworth',
            regexp: /^(Bessel$|Butterworth$|$)/
        }],
        filter_type: [{
            level: 'error',
            message_tr: 'Filter type must be Lowpass or Off',
            regexp: /^(Lowpass$|Off$|$)/
        }],
        max_signal_level: [{
            level: 'error',
            message_tr: 'Maximum signal level must be 100, 8000, 10200, or 30000 mv',
            regexp: /^(30000$|10200$|8000$|100$|$)/
        }],
        sensor_type: [{
            level: 'error',
            message_tr: 'Sensor type must be Bridge, Voltage, Pt, Resistor, Current, or IEPE',
            regexp: /^(Bridge$|Voltage$|Pt$|Resistor$|Current$|IEPE$|$)/
        }],
        transducer_power: [{
            level: 'error',
            message_tr: 'Transducer power must be numeric',
            regexp: /^(\s*(\d+|\d+\s*\/\s*\d+|\d+\.\d+)\s*$|$)/
        }],
        excitation_amplitude: [{
            level: 'error',
            message_tr: 'Excitation amplitude must be numeric',
            regexp: /^(\s*(\d+|\d+\s*\/\s*\d+|\d+\.\d+)\s*$|$)/
        }],
        wiring: [{
            level: 'error',
            message_tr: 'Wiring must be one of 3, 4, 5, or 6',
            regexp: /^(3$|4$|5$|6$|$)/
        }],
        excitation_frequency: [{
            level: 'error',
            message_tr: 'Excitation frequency must be one of Auto, DC, 1200Hz, or AutoCF',
            regexp: /^(Auto$|DC$|1200Hz$|AutoCF$|$)/
        }],
        bridge_type: [{
            level: 'error',
            message_tr: 'Bridge type must be one of Full, Half, or Quarter',
            regexp: /^(Full$|Half$|Quarter$|$)/
        }],
        bridge_resistance: [{
            level: 'error',
            message_tr: 'Bridge resistance must be 350 or 1000',
            regexp: /^(350$|1000$|$)/
        }],
        drift_compensation_mode: [{
            level: 'error',
            message_tr: 'Drift compensation mode must be one of CalOff, AutoAdjust, AutoCal, or Default',
            regexp: /^(CalOff$|AutoAdjust$|AutoCal$|Default$|$)/
        }],
        drift_repetition_time: [{
            level: 'error',
            message_tr: 'Drift compensation repitition time  must be numeric',
            regexp: /^(\s*(\d+|\d+\s*\/\s*\d+|\d+\.\d+)\s*$|$)/
        }],
        scaling_type: [{
            level: 'error',
            message_tr: 'Scaling type must be one of Off, Internal, ZeroSpan, LinTable, Gain, or Straingage',
            regexp: /^(Off$|Internal$|ZeroSpan$|LinTable$|Gain$|Straingage$|$)/
        }]
    };

    $.widget("ui.armada_grid", {
        options: {
            columns: null,
            slickgrid_options: null,
            data_view: null,
            column_set: null,
            column_picker: false,
            validate: false
        },

        grid: null,
        view: null,
        column_picker: null,
        all_columns: null,
        column_index: {},
        filter_string: "",
        filter_columns: [],
        local_edit_item: null,

        _create: function() {
            var options = this.options;
            var view = this.view = (options.data_view == null) ?
                new Slick.Data.ArmadaSetupModel() : options.data_view;
            var i;
            this.element.addClass("armada-grid");
            this.all_columns = options.columns; // KLUDGE attach to 'options' call
            for (i = 0; i < this.all_columns.length; i++)
                this.column_index[this.all_columns[i].id] = this.all_columns[i];

            var grid = this.grid = new Slick.Grid(this.element, view,
                options.columns, options.slickgrid_options);

            if (options.column_picker)
                this.column_picker = new Slick.Controls.ColumnPicker(
                    options.columns, grid, options.slickgrid_options);

            var validate = this.options.validate;

            function server_update_columns(args) {
                // KLUDGE must do authentication
                var server_updates = [];
                if (args.item) {
                    var columns = grid.getColumns();
                    if (backchannel_fields[columns[args.cell].field])
                        server_updates.push(args.item);
                }
                if (server_updates.length > 0) {
                    var req = {updates: server_updates};
                    $.ajax({
                        type: 'POST',
                        url: '/comet/update_settings/',
                        processData: false,
                        contentType: 'application/json; charset=utf-8',
                        data: $.toJSON(req),
                        cache: false,
                        success: function(data, status, xhr) {
                        },
                        error: function(data, status, xhr) {
                        }
                    });
                }
            }

            var that = this;

            grid.onBeforeEditCell.subscribe(function(e, args) {
                that.local_edit_item = _.clone(args.item);
                return true;
            });

            grid.onCellChange.subscribe(function(e, args) {
                server_update_columns(args);
                // We do this to kludge around SlickGrid's editing
                // in-place editing while we need deltas.
                var d_old = {};
                d_old[that.local_edit_item.id] = that.local_edit_item;
                var d_new = {};
                d_new[args.item.id] = args.item;
                view.deltaItems({old: d_old, cur: d_old, new: d_new});
            });

            grid.onContextMenu.subscribe(function(e) {
                e.preventDefault();
                var cell = grid.getCellFromEvent(e);
                $("#context_menu") /* HIDEOUS KLUDGE - use jQuery-UI menu */
                    .data("row", cell.row)
                    .css("top", e.pageY)
                    .css("left", e.pageX)
                    .one("click", function(e){
                        if (!$(e.target).is("li"))
                            return;
                        var row = $(this).data("row");
                        var action = $(e.target).attr("data");

                        var item = view.getItemByIdx(row);
                        if (action === "delete_row" && item && item.id != null)
                            view.deleteItem(item.id);
                    })
                    .show();
                $("body").one("click", function() {
                    $("#context_menu").hide();
                });
            });

            view.onRowCountChanged.subscribe(function(e, args) {
                grid.updateRowCount();
                grid.render();
            });

            view.onRowsChanged.subscribe(function(e, args) {
                grid.invalidateRows(args.rows);
                if (validate) {
                    that.validate(args);
                }
                grid.render();
            });

            view.onDelta.subscribe(function(e, delta) {
                if (validate) {
                    that.validate();
                }
                grid.invalidate(); // KLUDGE not specific enough
            });

            function filter(item, args) {
                if (args.filter_string.length === 0)
                    return true;
                for (var i = args.filter_columns.length - 1; i >= 0; i--) {
                    var v = item[args.filter_columns[i].field];
                    if (typeof(v) === "string" &&
                        v.indexOf(args.filter_string) !== -1)
                        return true;
                }
                return false;
            }

            view.setFilterArgs(this);
            view.setFilter(filter);
        },

        _destroy: function() {
        },

        _setOption: function(key, value) {
            this._super("_setOption", key, value);
            if (key === "column_set") {
                var columns = value.columns;
                if (columns == null)
                    columns = this.options.columns; // KLUDGE
                this.filter_columns = [];
                for (var i = 0; i < columns.length; i++) {
                    var spec = columns[i];
                    var col = null;
                    switch (typeof(spec)) {
                    case "string":
                        if (this.column_index[spec] != null)
                            col = $.extend({}, this.column_index[spec]);
                        break;
                    case "object":
                        if (this.column_index[spec.id] != null)
                            col = $.extend({}, this.column_index[spec.id], spec);
                        break;
                    }
                    if (col !== null) // KLUDGE ERROR
                        this.filter_columns.push(col);
                }
                this.grid.setColumns(this.filter_columns);
                this.validate(); // KLUDGE onColumnChange event???
            }
            if (key === "filter_string") {
                this.filter_string = value;
                this.view.setFilterArgs(this);
                this.view.refresh();
            }
        },

        innards: function(name) {
            return this[name];
        },

        validate: function(args) {
            var error_cell_map = {}; // [row]{name: pri, sample_rate: pri}
            var error_list = []; // KLUDGE just use error_view SlickGrid view ?
            var item_count = this.view.getLength();
            var items = this.view.getItems();
            var columns = this.grid.getColumns();
            var row, field;
            var counts = {
                error: 0,
                warning: 0,
                notice: 0
            };
            for (var i = 0; i < item_count; i++) {
                var item = items[i];
                var level_checks = cell_checks;
                for (field in level_checks) {
                    var checks = level_checks[field];
                    var checks_len = checks.length;
                    for (var check_idx = 0; check_idx < checks_len; check_idx++) {
                        var check = checks[check_idx];
                        var err = false;
                        var v = item[field];
                        if (v == null)
                            v = "";
                        if (check.regexp && !(check.regexp.test(v)))
                            err = true;
                        if (err) {
                            row = this.view.getRowById(item.id);
                            var this_err = $.extend({}, check, {row: row});
                            error_list.push(this_err); // KLUDGE formatting?
                            var pri = error_priorities[check.level];
                            if (error_cell_map[row] == null)
                                error_cell_map[row] = {};
                            var old_pri = error_cell_map[row][field];
                            error_cell_map[row][field] = (old_pri == null) ?
                                pri : Math.min(pri, old_pri);
                            ++counts[check.level];
                        }
                    }
                }
            }
            var slick_error_map = {};
            for (row in error_cell_map) {
                slick_error_map[row] = {};
                var row_errors = error_cell_map[row];
                var min_pri = 99999;
                for (field in row_errors) {
                    if (min_pri > row_errors[field])
                        min_pri = row_errors[field];
                }
                for (var j = columns.length - 1; j >= 0; j--) {
                    field = columns[j].field;
                    slick_error_map[row][field] = error_classes[
                        min_pri * 10 + (row_errors[field] === min_pri ? 0 : 1)
                    ];
                }
            }
            for (var level in counts) {
                var op = (counts[level]) ? 'removeClass' : 'addClass';
                $(".armada-" + level + "-count")
                    .text(counts[level])[op]('ui-state-disabled');
                // KLUDGE make this work with ui-state-disabled ???
                $(".armada-" + level + "-icon")[op]('armada-disabled');
            }
            this.grid.setCellCssStyles("armada-errors", slick_error_map);
        }
    });

    $.extend($.ui.armada_grid, {
        version: "@VERSION"
    });

}(jQuery));
